# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.forms import inlineformset_factory
from django.utils.translation import ugettext_lazy as _

from .models import Actor, Cast, Contact, Movie

class ActorForm(forms.ModelForm):
	class Meta:
		fields = '__all__'
		model  = Actor

class ContactForm(forms.ModelForm):
	class Meta:
		exclude = ('actor', )
		model = Contact

class MovieForm(forms.ModelForm):
	class Meta:
		fields = '__all__'
		model  = Movie

class CastForm(forms.ModelForm):
	class Meta:
		fields = '__all__'
		model  = Cast

"""
Formsets
"""
MovieCastFormSet = inlineformset_factory(
	Movie,
	Cast,
	form       = CastForm,
	extra      = 3,
	can_delete = True
)


# -*- coding: utf-8 -*-
"""
Django settings for demo project.

For more information on this file, see
https://docs.djangoproject.com/en/1.9/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.9/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
from django.core.urlresolvers import reverse_lazy

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.8/howto/deployment/checklist/

DEBUG = True

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '__SHHH_ITS_A_SECRET__'

ALLOWED_HOSTS = []

ADMINS = []

MANAGERS = []

INTERNAL_IPS = []

# Application definition
INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.humanize',
    # To make it look nice
    'bootstrap3',
    # Boilerplate
    'boilerplate',
    # Apps
    'account',
    'store',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.BrokenLinkEmailsMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'demo.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(BASE_DIR, 'templates/'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.core.context_processors.media',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

TEMPLATE_LOADERS = [
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader'
]

LOCALE_PATHS = [
    os.path.join(BASE_DIR, 'locale'),
]

WSGI_APPLICATION = 'demo.wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.9/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'


# Internationalization
# https://docs.djangoproject.com/en/1.8/topics/i18n/
LANGUAGE_CODE = 'en'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files
MEDIA_ROOT = os.path.join(BASE_DIR, 'media/')

STATIC_ROOT = os.path.join(BASE_DIR, 'static/')

MEDIA_URL = '/media/'

STATIC_URL = '/static/'

LOGIN_URL = reverse_lazy('account:login')



# -*- coding: utf-8 -*-
from django.core.exceptions import PermissionDenied
from django.contrib import messages
from django.db import IntegrityError, transaction
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _


class NoLoginRequiredMixin(object):
    """
    Mixin for any class view classes that required the current user
    if not authenticated, redirects the user to the home page or any
    URL with the *next* parameter.

    **Example**
    ::
        class Register(NoLoginRequiredMixin, FormView):
            form_class = forms.RegisterForm
    """

    def dispatch(self, request, *args, **kwargs):
        has_access = request.user.is_authenticated

        if (
            (callable(has_access) and has_access()) or
            (not callable(has_access) and has_access)
        ):
            raise PermissionDenied

        return super(NoLoginRequiredMixin, self).dispatch(
            request, *args, **kwargs
        )


class ActionListMixin(object):
    """
    Mixin for :class:`~django.views.generic.list.ListView` classes that
    adds to the "context" a variable with a list of actions.

    **Example**
    ::
        class ModelList(ActionListMixin, ListView):
            action_list = (
                _('Add'), 'create', 'primary', 'plus'),
                _('Export'), 'export', 'success', 'export'),
            )
            model = Model
    """
    def get_action_list(self):
        """
        Return the list of actions to show in the "context".

        Override this function to personalize the list of actions.

        For example: based on the user permissions.
        """
        return self.action_list

    def get_context_data(self, **kwargs):
        if 'actions' not in kwargs:
            kwargs['action_list'] = self.get_action_list()

        return super(ActionListMixin, self).get_context_data(**kwargs)


class UserCreateMixin(object):
    field_user = 'user'

    def get_field_user(self):
        return self.field_user

    def form_valid(self, form):
        setattr(form.instance, self.get_field_user(), self.request.user)

        return super(UserCreateMixin, self).form_valid(form)


class CRUDMessageMixin(object):
    """
    Add a message on successful form submission.
    """
    message_action = None
    success_message = _(
        '%(model)s: "%(name)s" has been %(action)s successfully.'
    )

    def form_valid(self, form):
        response = super(CRUDMessageMixin, self).form_valid(form)

        success_message = self.get_success_message(form.cleaned_data)

        if success_message:
            messages.success(self.request, success_message)

        return response

    def get_success_message(self, cleaned_data=None):
        """
        Default success message mixin
        """
        return self.success_message % dict(
            model=self.object.__class__._meta.verbose_name.title(),
            name=self.object,
            action=self.message_action,
        )


class CreateMessageMixin(CRUDMessageMixin):
    """
    Mixin for :class:`~django.views.generic.edit.CreateView` classes that
    adds a success message after the action is completed successfully.

    **Example**
    ::
        class ModelSendEmail(CreateMessageMixin, CreateView):
            model = Model
    """
    message_action = 'created'


class UpdateMessageMixin(CRUDMessageMixin):
    """
    Mixin for :class:`~django.views.generic.edit.UpdateView` classes that
    adds a success message after the action is completed successfully.

    **Example**
    ::
        class ModelSendEmail(UpdateMessageMixin, UpdateView):
            model = Model
    """
    message_action = 'updated'


class DeleteMessageMixin(CRUDMessageMixin):
    """
    Mixin for :class:`~django.views.generic.edit.DeleteView` classes that
    adds a success message after the action is completed successfully.

    **Example**
    ::
        class ModelSendEmail(DeleteMessageMixin, DeleteView):
            model = Model
    """
    message_action = 'deleted'

    def delete(self, request, *args, **kwargs):
        response = super(DeleteMessageMixin, self).delete(
            request, *args, **kwargs
        )

        success_message = self.get_success_message()

        if success_message:
            messages.success(self.request, success_message)

        return response


class ExtraFormsAndFormsetsMixin(object):
    """
    Mixin for :class:`~django.views.generic.edit.CreateView` or
    :class:`~django.views.generic.edit.UpdateView` classes that
    adds extra forms and formsets to any of thoose views.

    **Example**
    ::
        class UserUpdate(ExtraFormsAndFormsetsMixin, UpdateView):
            model = User
            extra_form_list = (
                ('profile', 'user', forms.ProfileForm),
                ('credit_card', 'user', forms.ProfileForm),
            )
            formset_list = (
                (form.AddressFormSet),
                (form.PhoneFormSet),
            )
    """
    extra_form_list = None
    formset_list = None

    def get_extra_form_list(self):
        """
        Returns a list of extra forms with the lookup_field,
        relation_field and form_class to use in this view.
        """
        return self.extra_form_list

    def get_formset_list(self):
        """
        Returns a list of formsets with the formset class to use in this view.
        """
        return self.formset_list

    def get_extra_forms(self, form_list=None):
        """
        Returns a list of each extra forms to be used in this view.
        """
        output = list()

        if form_list is None:
            form_list = self.get_extra_form_list()

        if form_list:
            for lookup_field, relation_field, extra_form in form_list:
                output.append(
                    (
                        relation_field,
                        extra_form(**self.get_extra_form_kwargs(lookup_field))
                    )
                )

        return output

    def get_formsets(self, formset_list=None):
        """
        Returns a list of formsets to be used in this view.
        """
        output = list()

        if formset_list is None:
            formset_list = self.get_formset_list()

        if formset_list:
            for formset in formset_list:
                output.append(formset(**self.get_formset_kwargs()))

        return output

    def get_extra_form_kwargs(self, lookup_field):
        """
        Returns the keyword arguments for instantiating each extra form.
        """
        kwargs = {
            'prefix': lookup_field,
        }

        if self.request.method in ('POST', 'PUT'):
            kwargs.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })

        if hasattr(self, 'object'):
            try:
                kwargs.update({'instance': getattr(self.object, lookup_field)})
            except Exception:
                pass

        return kwargs

    def get_formset_kwargs(self):
        """
        Returns the keyword arguments for instantiating each formset.
        """
        kwargs = {}

        if self.request.method in ('POST', 'PUT'):
            kwargs.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })

        if hasattr(self, 'object'):
            kwargs.update({'instance': self.object})

        return kwargs

    def post(self, request, *args, **kwargs):
        """
        Handles POST requests, instantiating a form instance with the passed
        POST variables and then checked for validity.
        """
        try:
            self.object = self.get_object()
        except Exception:
            self.object = None

        form = self.get_form()
        extra_forms = self.get_extra_forms()
        formsets = self.get_formsets()

        if not form.is_valid():
            return self.form_invalid(form, extra_forms, formsets)

        for relation_field, extra_form in extra_forms:
            if not extra_form.is_valid():
                return self.form_invalid(form, extra_forms, formsets)

        for formset in formsets:
            if not formset.is_valid():
                return self.form_invalid(form, extra_forms, formsets)

        return self.form_valid(form, extra_forms, formsets)

    def form_valid(self, form, extra_forms=None, formsets=None):
        """
        If the form is valid, redirect to the supplied URL.
        """
        try:
            with transaction.atomic():
                response = super(
                    ExtraFormsAndFormsetsMixin, self
                ).form_valid(form)

                if not self.object:
                    raise IntegrityError

                for relation_field, extra_form in extra_forms:
                    setattr(extra_form.instance, relation_field, self.object)

                    if not extra_form.is_valid():
                        raise IntegrityError

                    extra_form.save()

                for formset in formsets:
                    formset.instance = self.object

                    if not formset.is_valid():
                        raise IntegrityError

                    formset.save()
        except Exception:
            return self.form_invalid(form, extra_forms, formsets)

        return response

    def form_invalid(self, form, extra_forms=None, formsets=None):
        """
        If the form or the extra forms are invalid, re-render the context
        data with the data-filled form and errors.
        """
        return self.render_to_response(
            self.get_context_data(
                form=form, extra_forms=extra_forms, formsets=formsets
            )
        )

    def get_context_data(self, **kwargs):
        """
        Insert the extra forms and formsets into the context dict.
        """
        if 'extra_form_list' not in kwargs:
            kwargs['extra_form_list'] = list()

            for relation_field, extra_form in self.get_extra_forms():
                kwargs['extra_form_list'].append(extra_form)
        else:
            extra_forms = list()

            for relation_field, extra_form in kwargs['extra_form_list']:
                extra_forms.append(extra_form)

            kwargs['extra_form_list'] = extra_forms

        if 'formset_list' not in kwargs:
            kwargs['formset_list'] = self.get_formsets()

        return super(
            ExtraFormsAndFormsetsMixin, self
        ).get_context_data(**kwargs)


class ParentMixin(object):
    """
    Mixin for :class:`~django.views.generic.edit.CreateView` or
    :class:`~django.views.generic.edit.UpdateView` classes that
    adds a success message after the action is completed successfully.

    **Example**
    ::
        class ProfileUpdate(ParentMixin, UpdateView):
            model = Profile
            parent_model = User
            parent_lookup_arg = 'pk_parent'
            parent_lookup_field = 'pk'
    """
    parent_model = None
    parent_lookup_arg = 'pk_parent'
    parent_lookup_field = 'pk'

    def get_parent_model(self):
        return self.parent_model

    def get_parent_lookup_arg(self):
        return self.parent_lookup_arg

    def get_parent_lookup_field(self):
        return self.parent_lookup_field

    def get_parent_kwargs(self):
        kwargs = {
            self.get_parent_lookup_field(): self.kwargs[
                self.get_parent_lookup_arg()
            ],
        }

        return kwargs

    def get_parent(self):
        return get_object_or_404(
            self.get_parent_model(), **self.get_parent_kwargs()
        )

    def get_context_data(self, **kwargs):
        if 'parent_object' not in kwargs:
            kwargs['parent_object'] = self.get_parent()

        return super(ParentMixin, self).get_context_data(**kwargs)


class ParentCreateMixin(ParentMixin):
    parent_relation_field = None

    def get_parent_relation_field(self):
        return self.parent_relation_field

    def form_valid(self, form):
        setattr(
            form.instance, self.get_parent_relation_field(), self.get_parent()
        )

        return super(ParentCreateMixin, self).form_valid(form)


class ParentCreateExtraMixin(ParentCreateMixin):
    def form_valid(self, form, extra_forms, formsets):
        setattr(form.instance, self.get_parent_set_field(), self.get_parent())

        return super(ParentCreateExtraMixin, self).form_valid(
            form, extra_forms, formsets
        )


class ParentSingleObjectMixin(ParentMixin):
    parent_relation_field = None

    def get_parent_relation_field(self):
        return self.parent_relation_field

    def get_queryset(self):
        qs = super(ParentSingleObjectMixin, self).get_queryset()

        kwargs = {
            self.get_parent_relation_field(): self.get_parent(),
        }

        return qs.filter(**kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template


class SendEmail(object):
    """
    Send an emails or several emails easily.

    **Example**
    ::
        email = SendEmail(
            to='user@example.com',
            template_name_suffix='recover-account',
            subject='Recover your account',
            is_html=True
        )
        email.set_from_email('no-reply@example.com')
        email.set_from_name('No Reply')
        email.add_context_data('protocol', 'https')
        email.add_context_data('domain', 'example.com')
        email.add_context_data('uid', uid)
        email.add_context_data('token', token)
        email.add_context_data('object', user)
        email.add_context_data('site_name', 'Boilerplate - Make it easy')
        email.send()
    """

    context_data = {}
    files = list()
    from_name = None
    from_email = None
    template_name = None
    content = None

    def __init__(
        self, to, subject, is_html=False, template_name_suffix=None,
        content=None
    ):
        if isinstance(to, list):
            self.to = to
        else:
            self.to = list()
            self.to.append(to)

        self.content = content
        self.template_name_suffix = template_name_suffix
        self.subject = subject
        self.is_html = is_html

    def add_file(self, file):
        self.files.append(file)

    def add_context_data(self, key, value):
        """
        Add a key-value to the context data of the email

        **Parameters**:
            :key: A valid key name
            :value: Can be an object or any kind of value
        """
        self.context_data.update({
            str(key): value
        })

    def set_from_email(self, from_email=None):
        """
        Set the email sender

        **Parameters**:
            :from_email: String, a valid email
        """
        if from_email:
            self.from_email = str(from_email)
        else:
            self.from_email = settings.SERVER_EMAIL

    def set_from_name(self, from_name=None):
        """
        Set the name sender

        **Parameters**:
            :from_name: String, a valid name
        """
        if from_name:
            self.from_name = str(from_name)
        else:
            self.from_name = settings.SERVER_EMAIL

    def set_template_name_suffix(self, template_name_suffix=None):
        """
        Set the email template name suffix

        **Parameters**:
            :template_name_suffix: String: the name of the template without the
            extension
        """
        if template_name_suffix:
            self.template_name_suffix = str(template_name_suffix)

    def set_template_name(self, template_name=None):
        """
        Set the email template name

        **Parameters**:
            :template_name: String: the name of the template without the
            extension
        """
        if template_name:
            self.template_name = template_name
        else:
            if not self.template_name_suffix:
                self.set_template_name_suffix()

            if self.template_name_suffix:
                self.template_name = 'mail/' + self.template_name_suffix
            else:
                self.template_name = None

    def get_content(self):
        return self.content

    def get_from_email(self):
        if not self.from_email:
            self.set_from_email()

        return self.from_email

    def get_from_name(self):
        if not self.from_name:
            self.set_from_name()

        return self.from_name

    def get_template_name_suffix(self):
        if not self.template_name_suffix:
            self.set_template_name_suffix()

        return self.template_name_suffix

    def get_template_name(self):
        if not self.template_name:
            self.set_template_name()

        return self.template_name

    def get_context_data(self, **kwargs):
        self.context_data.update({
            'email': self
        })

        return self.context_data

    def send(self, fail_silently=True, test=False):
        template_name = self.get_template_name()
        content = self.get_content()

        if not template_name and not content:
            raise Exception(
                "You need to set the `template_name_suffix` or `content`."
            )

        if template_name:
            plain_template = get_template(template_name + '.txt')
            plain_content = plain_template.render(self.get_context_data())

            if self.is_html:
                html_template = get_template(template_name + '.html')
                html_content = html_template.render(self.get_context_data())
        elif content:
            plain_content = content

            if self.is_html:
                html_content = content

        if test:
            return plain_content

        if self.is_html:
            content = html_content
        else:
            content = plain_content

        msg = EmailMultiAlternatives(
            subject=self.subject,
            body=content,
            from_email='%s <%s>' % (
                self.get_from_name(), self.get_from_email()
            ),
            to=self.to
        )

        if self.files:
            for name, file, content_type in self.files:
                try:
                    msg.attach(name, file.read(), content_type)
                except Exception:
                    msg.attach(name, file.getvalue(), content_type)

        return msg.send(fail_silently=fail_silently)



from django.core.urlresolvers import reverse_lazy
from boilerplate.views import (ChangePasswordView, LoginView, LogoutView, 
	RecoverAccountView, RecoverAccountConfirmView, RegistrationView)

class ChangePassword(ChangePasswordView):
	template_name = 'account/password.html'

class Login(LoginView):
	template_name = 'account/login.html'

class Logout(LogoutView):
	success_url = reverse_lazy('store:movie_list')

class Recover(RecoverAccountView):
	recover_url   = 'account:recover_confirm'
	success_url   = reverse_lazy('store:movie_list')
	template_name = 'account/recover.html'

class RecoverConfirm(RecoverAccountConfirmView):
	success_url   = reverse_lazy('account:login')
	template_name = 'account/recover_confirm.html'

class Registration(RegistrationView):
	success_url   = reverse_lazy('store:movie_list')
	template_name = 'account/registration.html'


# -*- coding: utf-8 -*-
import sys

from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType


def add_view_permissions(sender, **kwargs):
    """
    Create a view permission for all the models in an specific app

    **Example**
    ::
    # File: apps.py
    from django.db.models.signals import post_migrate
    from boilerplate.signals import add_view_permissions

    class MyAppConfig(AppConfig):
        ...

        def ready(self):
            post_migrate.connect(add_view_permissions, sender=self)
    ::
    """
    for content_type in ContentType.objects.filter(app_label=sender.label):
        codename = "view_%s" % content_type.model

        perm, created = Permission.objects.get_or_create(
            content_type=content_type, codename=codename, defaults={
                'name': 'Can view %s' % content_type.name,
            }
        )

        if created:
            sys.stdout.write(
                'Added view permission for %s' % content_type.name +
                '\n'
            )
